/*  sensor_applet

    Copyright (C) 2001 Osamu Kayasono <jacobi@jcom.home.ne.jp>

    This program is GNOME applet for reading lm_sensors statistics.
	Please read README for more details.
*/

#include "sensor_applet.h"

gint main(int argc, char ** argv)
{
   SensorApplet *ai;
   gint i,j;
   char chips[STRMAX];

   gtk_set_locale();
   gtk_init(&argc, &argv);

   if (find_chips(chips) == 1) {
      printf( _("%s : lm_sensors is not activate\n"),PACKAGE);
      exit(1);
   }

   ai = g_malloc(sizeof(SensorApplet));
   ai->chips = chips;
   for (i = 1; i <= 5; i++) {
      for (j = 0;  j< 52; j++) {
	 ai->value[i][j] = 0;
      }
   }

   applet_widget_init(PACKAGE, VERSION, argc, argv, NULL, 0, NULL);
   /*  gdk_imlib_init(); */
   readPixmaps();
   
   ai->applet = applet_widget_new(PACKAGE);
   if(!ai->applet) g_error(_("Can't create sensor_applet!"));
   gtk_widget_realize(ai->applet);

   ai->canvas = gnome_canvas_new ();
   gtk_widget_set_usize (ai->canvas, AREA_WIDTH,AREA_HEIGHT);
   applet_widget_add (APPLET_WIDGET (ai->applet), ai->canvas);
 
   /* Initialize the Gnome config system */
   property_load(APPLET_WIDGET(ai->applet)->privcfgpath,ai);
   gtk_widget_set_usize(ai->applet,ai->width,ai->height);

   /* Create and setup the event box widget */
   /*  ai->eventbox = gtk_event_box_new();
    applet_widget_add(APPLET_WIDGET(ai->applet), ai->eventbox);
    gtk_widget_set_events(ai->eventbox, GDK_BUTTON_PRESS_MASK); */

   gtk_signal_connect (GTK_OBJECT (ai->applet),
		       "save_session",GTK_SIGNAL_FUNC(save_session_cb),ai);
   gtk_signal_connect(GTK_OBJECT(ai->applet), "delete_event",
		      GTK_SIGNAL_FUNC(delete_event), NULL);  

   applet_widget_register_stock_callback (APPLET_WIDGET (ai->applet),
	 "Properties", GNOME_STOCK_MENU_PROP, _("Properties"),
	 (AppletCallbackFunc)properties, ai);
   applet_widget_register_stock_callback (APPLET_WIDGET (ai->applet),
	 "about", GNOME_STOCK_MENU_ABOUT, _("About"),
	 (AppletCallbackFunc)about_cb, ai);
   /*  applet_widget_register_stock_callback (APPLET_WIDGET (ai->applet),
    "quit", GNOME_STOCK_MENU_ABOUT, _("Quit"),
    quit_applet, ai); */

  /*	gtk_signal_connect(GTK_OBJECT(ai->eventbox), "button_press_event", GTK_SIGNAL_FUNC(updateGUIWrapper), ai); */

   for(i = 0; i <= 8; i++) {
      for(j=0 ; j <= 4; j++) {
	 ai->lt[i][j] = gnome_canvas_item_new(
		   gnome_canvas_root(GNOME_CANVAS(ai->canvas)),
		   gnome_canvas_image_get_type (), "image", letter_image[13],
		   "x", (double) (CHAR_WIDTH)*(i+1),
		   "y", (double) (CHAR_HEIGHT+1)*(j+1),
		   "width", (double) CHAR_WIDTH,
		   "height", (double) (CHAR_HEIGHT),  NULL);
      }
   }

   /* Show the widget's */
   gtk_widget_show(ai->canvas);
   gtk_widget_show(ai->applet);
        
   update_val(ai);

   ai->timeout_id = gtk_timeout_add(ai->sensorFreq * 1000, 
				    (GtkFunction)update_val, ai);

   applet_widget_gtk_main();

   return (0);
}

gint update_val(SensorApplet *ai)
{
  gchar msg[10];
  gchar letters[]="+-/.0123456789 ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  gchar *letter;
  gint i,j;
  gboolean warn;

  /*  read_proc(chips); */
  read_sensors(ai);

  for (i = 1; i <= 5; i++) {
    warn=FALSE;

    if (strlen(ai->label[i]) > 4) break;
    if (ai->value[i][0] >= 100) {
      sprintf(msg,"%-4s %4d",ai->label[i],(int)ai->value[i][0]);
    } else {
      if (ai->value[i][0] >= 10) {
	sprintf(msg,"%-4s %2.1f",ai->label[i],ai->value[i][0]);
      } else {
	if ((ai->value[i][0] == 0) || (ai->value[i][0] == DISABLE)) {
	  sprintf(msg,"%-4s ----",ai->label[i]);
	} else {
	  sprintf(msg,"%-4s %2.2f",ai->label[i],ai->value[i][0]);
	}
      }
    }
    warn = check_warn(i,ai->sensor[i],ai);

    for (j=0;j<strlen(msg);j++) {
      letter=strchr(letters,toupper(msg[j]));
      if(letter != NULL) {
	if (!warn) {
	  gnome_canvas_item_set (ai->lt[j][i-1], "image", 
				 letter_image[letter-letters], NULL);
	} else {
	  gnome_canvas_item_set (ai->lt[j][i-1], "image", 
				 warn_image[letter-letters], NULL);
	}
      }
    }
  }
 return TRUE;
}

static gboolean check_warn(gint i, gchar *path, SensorApplet *ai)
{
  /* Fan parameters are current value and low limit */
  if (strncmp(path,"fan",3) == 0) {
      if (ai->value[i][0] !=0 && ai->value[i][0] != DISABLE && 
	  ai->value[i][0] < ai->min[i]) return TRUE;
  }

  /* Temp. parameters are current value and high limit */
  if (strncmp(path,"temp",4) == 0) {
      if (ai->value[i][0] !=0 && ai->value[i][0] != DISABLE && 
	  ai->value[i][0] > ai->max[i]) return TRUE;
  }

  /* Voltages */
  if (ai->value[i][0] !=0 && ai->value[i][0] != DISABLE) {
    if (ai->value[i][0] < ai->min[i] && ai->min[i] != DISABLE) return TRUE;
    if (ai->value[i][0] > ai->max[i] && ai->max[i] != DISABLE) return TRUE;
  }
  return FALSE;
}

static gint read_sensors(SensorApplet *ai)
{
  gint i;

  for (i=1;i<=5;i++) {
    memmove(&ai->value[i][1], &ai->value[i][0], sizeof(float)*51); 
    ai->value[i][0] = read_val(ai->chips,ai->sensor[i],0);
    ai->min[i] = read_val(ai->chips,ai->sensor[i],-1);
    ai->max[i] = read_val(ai->chips,ai->sensor[i],1);
  }
  return (0);
}

static void readPixmaps(){
  gint i;
  GtkStyle *style;

  style = gtk_widget_get_default_style();

  for(i = 0; i <= 39; i++) { 
    letter_image[i] = gdk_imlib_create_image_from_xpm_data(letter_xpm[i]);
    letter_xpm[i][3]="B	c red";
    warn_image[i] = gdk_imlib_create_image_from_xpm_data(letter_xpm[i]);
  }
  return;
}
